@@ -83,12 +83,12 @@ static deh_context_t *DEH_NewContext(void)
 // Open a dehacked file for reading
 // Returns NULL if open failed
 
-deh_context_t *DEH_OpenFile(char *filename)
+deh_context_t *DEH_OpenFile(const char *filename)
 {
     FILE *fstream;
     deh_context_t *context;
 
-    fstream = fopen(filename, "r");
+    fstream = M_fopen(filename, "r");
 
     if (fstream == NULL)
         return NULL;
@@ -175,9 +175,16 @@ int DEH_GetCharLump(deh_context_t *context)
 int DEH_GetChar(deh_context_t *context)
 {
     int result = 0;
+    boolean last_was_cr = false;
 
-    // Read characters, but ignore carriage returns
-    // Essentially this is a DOS->Unix conversion
+    // Track the current line number
+
+    if (context->last_was_newline)
+    {
+        ++context->linenum;
+    }
+
+    // Read characters, converting CRLF to LF
 
     do
     {
@@ -191,14 +198,27 @@ int DEH_GetChar(deh_context_t *context)
                 result = DEH_GetCharLump(context);
                 break;
         }
-    } while (result == '\r');
 
-    // Track the current line number
+        // Handle \r characters not paired with \n
+        if (last_was_cr && result != '\n')
+        {
+            switch (context->type)
+            {
+                case DEH_INPUT_FILE:
+                    ungetc(result, context->stream);
+                    break;
 
-    if (context->last_was_newline)
-    {
-        ++context->linenum;
-    }
+                case DEH_INPUT_LUMP:
+                    --context->input_buffer_pos;
+                    break;
+            }
+
+            return '\r';
+        }
+
+        last_was_cr = result == '\r';
+
+    } while (last_was_cr);
 
     context->last_was_newline = result == '\n';
 
@@ -302,7 +322,7 @@ char *DEH_ReadLine(deh_context_t *context, boolean extended)
     return context->readbuffer;
 }
 
-void DEH_Warning(deh_context_t *context, char *msg, ...)
+void DEH_Warning(deh_context_t *context, const char *msg, ...)
 {
     va_list args;
 
@@ -315,7 +335,7 @@ void DEH_Warning(deh_context_t *context, char *msg, ...)
     va_end(args);
 }
 
-void DEH_Error(deh_context_t *context, char *msg, ...)
+void DEH_Error(deh_context_t *context, const char *msg, ...)
 {
     va_list args;
 
