@@ -20,13 +20,13 @@
 
 #include "deh_defs.h"
 
-deh_context_t *DEH_OpenFile(char *filename);
+deh_context_t *DEH_OpenFile(const char *filename);
 deh_context_t *DEH_OpenLump(int lumpnum);
 void DEH_CloseFile(deh_context_t *context);
 int DEH_GetChar(deh_context_t *context);
 char *DEH_ReadLine(deh_context_t *context, boolean extended);
-void DEH_Error(deh_context_t *context, char *msg, ...);
-void DEH_Warning(deh_context_t *context, char *msg, ...);
+void DEH_Error(deh_context_t *context, const char *msg, ...) PRINTF_ATTR(2, 3);
+void DEH_Warning(deh_context_t *context, const char *msg, ...) PRINTF_ATTR(2, 3);
 boolean DEH_HadError(deh_context_t *context);
 
 #endif /* #ifndef DEH_IO_H */
