@@ -20,13 +20,15 @@
 
 #include <stdio.h>
 
+#include "doomtype.h"
+
 // Used to do dehacked text substitutions throughout the program
 
-char *DEH_String(char *s);
-void DEH_printf(char *fmt, ...);
-void DEH_fprintf(FILE *fstream, char *fmt, ...);
-void DEH_snprintf(char *buffer, size_t len, char *fmt, ...);
-void DEH_AddStringReplacement(char *from_text, char *to_text);
+const char *DEH_String(const char *s) PRINTF_ARG_ATTR(1);
+void DEH_printf(const char *fmt, ...) PRINTF_ATTR(1, 2);
+void DEH_fprintf(FILE *fstream, const char *fmt, ...) PRINTF_ATTR(2, 3);
+void DEH_snprintf(char *buffer, size_t len, const char *fmt, ...) PRINTF_ATTR(3, 4);
+void DEH_AddStringReplacement(const char *from_text, const char *to_text);
 
 
 #if 0
