@@ -52,10 +52,11 @@ void M_Init (void);
 void M_StartControlPanel (void);
 
 
-
 extern int detailLevel;
 extern int screenblocks;
 
+extern boolean inhelpscreens;
+extern int showMessages;
 
 
-#endif    
+#endif
