@@ -20,6 +20,7 @@
 
 
 #include <math.h>
+#include <stdlib.h>
 
 #include "z_zone.h"
 
@@ -107,6 +108,7 @@ byte*		rejectmatrix;
 mapthing_t	deathmatchstarts[MAX_DEATHMATCH_STARTS];
 mapthing_t*	deathmatch_p;
 mapthing_t	playerstarts[MAXPLAYERS];
+boolean     playerstartsingame[MAXPLAYERS];
 
 
 
@@ -389,6 +391,18 @@ void P_LoadThings (int lump)
 	P_SpawnMapThing(&spawnthing);
     }
 
+    if (!deathmatch)
+    {
+        for (i = 0; i < MAXPLAYERS; i++)
+        {
+            if (playeringame[i] && !playerstartsingame[i])
+            {
+                I_Error("P_LoadThings: Player %d start missing (vanilla crashes here)", i + 1);
+            }
+            playerstartsingame[i] = false;
+        }
+    }
+
     W_ReleaseLumpNum(lump);
 }
 
@@ -701,7 +715,7 @@ static void PadRejectArray(byte *array, unsigned int len)
 
     if (len > sizeof(rejectpad))
     {
-        fprintf(stderr, "PadRejectArray: REJECT lump too short to pad! (%i > %i)\n",
+        fprintf(stderr, "PadRejectArray: REJECT lump too short to pad! (%u > %i)\n",
                         len, (int) sizeof(rejectpad));
 
         // Pad remaining space with 0 (or 0xff, if specified on command line).
@@ -712,7 +726,7 @@ static void PadRejectArray(byte *array, unsigned int len)
         }
         else
         {
-            padvalue = 0xf00;
+            padvalue = 0x00;
         }
 
         memset(array + sizeof(rejectpad), padvalue, len - sizeof(rejectpad));
@@ -747,6 +761,9 @@ static void P_LoadReject(int lumpnum)
     }
 }
 
+// pointer to the current map lump info struct
+lumpinfo_t *maplumpinfo;
+
 //
 // P_SetupLevel
 //
@@ -803,6 +820,8 @@ P_SetupLevel
 
     lumpnum = W_GetNumForName (lumpname);
 	
+    maplumpinfo = lumpinfo[lumpnum];
+
     leveltime = 0;
 	
     // note: most of this ordering is important	
