@@ -40,12 +40,16 @@ void R_PrecacheLevel (void);
 // Retrieval.
 // Floor/ceiling opaque texture tiles,
 // lookup by name. For animation?
-int R_FlatNumForName (char* name);
+int R_FlatNumForName(const char *name);
 
 
 // Called by P_Ticker for switches and animations,
 // returns the texture number for the texture name.
-int R_TextureNumForName (char *name);
-int R_CheckTextureNumForName (char *name);
+int R_TextureNumForName(const char *name);
+int R_CheckTextureNumForName(const char *name);
+
+
+extern int numflats;
+
 
 #endif
