@@ -21,7 +21,6 @@
 #include "i_sound.h"
 #include "i_system.h"
 
-#include "doomfeatures.h"
 #include "deh_str.h"
 
 #include "doomstat.h"
@@ -662,7 +661,8 @@ void S_ChangeMusic(int musicnum, int looping)
     // and d_introa.  The latter is used for OPL playback.
 
     if (musicnum == mus_intro && (snd_musicdevice == SNDDEVICE_ADLIB
-                               || snd_musicdevice == SNDDEVICE_SB))
+                               || snd_musicdevice == SNDDEVICE_SB)
+        && W_CheckNumForName("D_INTROA") >= 0)
     {
         musicnum = mus_introa;
     }
