@@ -34,7 +34,7 @@ int ArmorIncrement[NUMCLASSES][NUMARMOR] = {
 int AutoArmorSave[NUMCLASSES] =
     { 15 * FRACUNIT, 10 * FRACUNIT, 5 * FRACUNIT, 0 };
 
-char *TextKeyMessages[] = {
+const char *TextKeyMessages[] = {
     TXT_KEY_STEEL,
     TXT_KEY_CAVE,
     TXT_KEY_AXE,
@@ -53,7 +53,7 @@ static void TryPickupArtifact(player_t * player, artitype_t artifactType,
                               mobj_t * artifact);
 static void TryPickupWeapon(player_t * player, pclass_t weaponClass,
                             weapontype_t weaponType, mobj_t * weapon,
-                            char *message);
+                            const char *message);
 static void TryPickupWeaponPiece(player_t * player, pclass_t matchClass,
                                  int pieceValue, mobj_t * pieceMobj);
 
@@ -63,7 +63,7 @@ static void TryPickupWeaponPiece(player_t * player, pclass_t matchClass,
 //
 //--------------------------------------------------------------------------
 
-void P_SetMessage(player_t * player, char *message, boolean ultmsg)
+void P_SetMessage(player_t * player, const char *message, boolean ultmsg)
 {
     if ((player->ultimateMessage || !messageson) && !ultmsg)
     {
@@ -90,7 +90,7 @@ void P_SetMessage(player_t * player, char *message, boolean ultmsg)
 //
 //==========================================================================
 
-void P_SetYellowMessage(player_t * player, char *message, boolean ultmsg)
+void P_SetYellowMessage(player_t * player, const char *message, boolean ultmsg)
 {
     if ((player->ultimateMessage || !messageson) && !ultmsg)
     {
@@ -190,7 +190,7 @@ boolean P_GiveMana(player_t * player, manatype_t mana, int count)
 
 static void TryPickupWeapon(player_t * player, pclass_t weaponClass,
                             weapontype_t weaponType, mobj_t * weapon,
-                            char *message)
+                            const char *message)
 {
     boolean remove;
     boolean gaveMana;
@@ -412,12 +412,12 @@ static void TryPickupWeaponPiece(player_t * player, pclass_t matchClass,
     boolean checkAssembled;
     boolean gaveWeapon;
     int gaveMana;
-    static char *fourthWeaponText[] = {
+    static const char *fourthWeaponText[] = {
         TXT_WEAPON_F4,
         TXT_WEAPON_C4,
         TXT_WEAPON_M4
     };
-    static char *weaponPieceText[] = {
+    static const char *weaponPieceText[] = {
         TXT_QUIETUS_PIECE,
         TXT_WRAITHVERGE_PIECE,
         TXT_BLOODSCOURGE_PIECE
@@ -567,8 +567,6 @@ boolean P_GiveArmor(player_t * player, armortype_t armortype, int amount)
     int hits;
     int totalArmor;
 
-    extern int ArmorMax[NUMCLASSES];
-
     if (amount == -1)
     {
         hits = ArmorIncrement[player->class][armortype];
@@ -711,7 +709,7 @@ boolean P_GivePower(player_t * player, powertype_t power)
 static void TryPickupArtifact(player_t * player, artitype_t artifactType,
                               mobj_t * artifact)
 {
-    static char *artifactMessages[NUMARTIFACTS] = {
+    static const char *artifactMessages[NUMARTIFACTS] = {
         NULL,
         TXT_ARTIINVULNERABILITY,
         TXT_ARTIHEALTH,
@@ -894,7 +892,7 @@ static void SetDormantArtifact(mobj_t * arti)
 //
 //---------------------------------------------------------------------------
 
-void A_RestoreArtifact(mobj_t * arti)
+void A_RestoreArtifact(mobj_t *arti, player_t *player, pspdef_t *psp)
 {
     arti->flags |= MF_SPECIAL;
     P_SetMobjState(arti, arti->info->spawnstate);
@@ -909,7 +907,7 @@ void A_RestoreArtifact(mobj_t * arti)
 //
 //---------------------------------------------------------------------------
 
-void A_RestoreSpecialThing1(mobj_t * thing)
+void A_RestoreSpecialThing1(mobj_t * thing, player_t *player, pspdef_t *psp)
 {
     thing->flags2 &= ~MF2_DONTDRAW;
     S_StartSound(thing, SFX_RESPAWN);
@@ -921,7 +919,7 @@ void A_RestoreSpecialThing1(mobj_t * thing)
 //
 //---------------------------------------------------------------------------
 
-void A_RestoreSpecialThing2(mobj_t * thing)
+void A_RestoreSpecialThing2(mobj_t * thing, player_t *player, pspdef_t *psp)
 {
     thing->flags |= MF_SPECIAL;
     P_SetMobjState(thing, thing->info->spawnstate);
@@ -2174,7 +2172,7 @@ void P_PoisonDamage(player_t * player, mobj_t * source, int damage,
     {                           // mobj is invulnerable
         return;
     }
-    if (player && gameskill == sk_baby)
+    if (gameskill == sk_baby)
     {
         // Take half damage in trainer mode
         damage >>= 1;
@@ -2203,7 +2201,7 @@ void P_PoisonDamage(player_t * player, mobj_t * source, int damage,
     if (target->health <= 0)
     {                           // Death
         target->special1.i = damage;
-        if (player && inflictor && !player->morphTics)
+        if (inflictor && !player->morphTics)
         {                       // Check for flame death
             if ((inflictor->flags2 & MF2_FIREDAMAGE)
                 && (target->health > -50) && (damage > 25))
