@@ -20,6 +20,8 @@
 #include "h2def.h"
 #include "i_system.h"
 #include "r_local.h"
+#include "p_spec.h"
+
 
 // MACROS ------------------------------------------------------------------
 
@@ -33,9 +35,6 @@
 
 // EXTERNAL DATA DECLARATIONS ----------------------------------------------
 
-extern fixed_t Sky1ScrollDelta;
-extern fixed_t Sky2ScrollDelta;
-
 // PUBLIC DATA DEFINITIONS -------------------------------------------------
 
 int Sky1Texture;
@@ -384,22 +383,20 @@ void R_DrawPlanes(void)
     int skyTexture2;
     int scrollOffset;
 
-    extern byte *ylookup[MAXHEIGHT];
-    extern int columnofs[MAXWIDTH];
-
 #ifdef RANGECHECK
     if (ds_p - drawsegs > MAXDRAWSEGS)
     {
-        I_Error("R_DrawPlanes: drawsegs overflow (%i)", ds_p - drawsegs);
+        I_Error("R_DrawPlanes: drawsegs overflow (%td)",
+                ds_p - drawsegs);
     }
     if (lastvisplane - visplanes > MAXVISPLANES)
     {
-        I_Error("R_DrawPlanes: visplane overflow (%i)",
+        I_Error("R_DrawPlanes: visplane overflow (%td)",
                 lastvisplane - visplanes);
     }
     if (lastopening - openings > MAXOPENINGS)
     {
-        I_Error("R_DrawPlanes: opening overflow (%i)",
+        I_Error("R_DrawPlanes: opening overflow (%td)",
                 lastopening - openings);
     }
 #endif
