@@ -27,6 +27,8 @@
 #include "s_sound.h"
 #include "v_video.h"
 #include "i_swap.h"
+#include "am_map.h"
+
 
 // TYPES -------------------------------------------------------------------
 
@@ -85,9 +87,6 @@ static void CheatTrackFunc2(player_t * player, Cheat_t * cheat);
 
 // EXTERNAL DATA DECLARATIONS ----------------------------------------------
 
-extern int ArmorIncrement[NUMCLASSES][NUMARMOR];
-extern int AutoArmorSave[NUMCLASSES];
-
 // PUBLIC DATA DECLARATIONS ------------------------------------------------
 
 boolean DebugSound;             // Debug flag for displaying sound info
@@ -740,7 +739,6 @@ static int oldpieces = -1;
 static int oldweapon = -1;
 static int oldkeys = -1;
 
-extern boolean automapactive;
 
 void SB_Drawer(void)
 {
@@ -1272,7 +1270,7 @@ void DrawKeyBar(void)
     if (oldkeys != CPlayer->keys)
     {
         xPosition = 46;
-        for (i = 0; i < NUMKEYS && xPosition <= 126; i++)
+        for (i = 0; i < NUM_KEY_TYPES && xPosition <= 126; i++)
         {
             if (CPlayer->keys & (1 << i))
             {
@@ -1639,7 +1637,6 @@ static void CheatNoClipFunc(player_t * player, Cheat_t * cheat)
 static void CheatWeaponsFunc(player_t * player, Cheat_t * cheat)
 {
     int i;
-    //extern boolean *WeaponInShareware;
 
     for (i = 0; i < NUMARMOR; i++)
     {
@@ -1775,8 +1772,6 @@ static void CheatWarpFunc(player_t * player, Cheat_t * cheat)
 
 static void CheatPigFunc(player_t * player, Cheat_t * cheat)
 {
-    extern boolean P_UndoPlayerMorph(player_t * player);
-
     if (player->morphTics)
     {
         P_UndoPlayerMorph(player);
@@ -1928,7 +1923,6 @@ static void CheatScriptFunc3(player_t * player, Cheat_t * cheat)
     }
 }
 
-extern int cheating;
 
 static void CheatRevealFunc(player_t * player, Cheat_t * cheat)
 {
