@@ -17,6 +17,7 @@
 // Use to convert a MUS file into a single track, type 0 MIDI file.
 
 #include <stdio.h>
+#include <stdlib.h>
 
 #include "doomtype.h"
 #include "i_swap.h"
@@ -53,7 +54,7 @@ typedef enum
 } midievent;
 
 // Structure to hold MUS file header
-typedef struct
+typedef PACKED_STRUCT (
 {
     byte id[4];
     unsigned short scorelength;
@@ -61,7 +62,7 @@ typedef struct
     unsigned short primarychannels;
     unsigned short secondarychannels;
     unsigned short instrumentcount;
-} PACKEDATTR musheader;
+}) musheader;
 
 // Standard MIDI type 0 header + track header
 static const byte midiheader[] =
