@@ -28,6 +28,7 @@
 #include "i_swap.h"
 #include "i_video.h"
 
+#include "d_main.h"
 #include "hu_stuff.h"
 #include "hu_lib.h"
 #include "m_controls.h"
@@ -58,19 +59,7 @@
 #define HU_INPUTWIDTH   64
 #define HU_INPUTHEIGHT  1
 
-char *chat_macros[10] =
-{
-    HUSTR_CHATMACRO0,
-    HUSTR_CHATMACRO1,
-    HUSTR_CHATMACRO2,
-    HUSTR_CHATMACRO3,
-    HUSTR_CHATMACRO4,
-    HUSTR_CHATMACRO5,
-    HUSTR_CHATMACRO6,
-    HUSTR_CHATMACRO7,
-    HUSTR_CHATMACRO8,
-    HUSTR_CHATMACRO9
-};
+char *chat_macros[10];
 
 // villsa [STRIFE]
 char player_names[8][16] =
@@ -103,13 +92,9 @@ static boolean          message_nottobefuckedwith;
 static hu_stext_t       w_message;
 static int              message_counter;
 
-//extern int              showMessages; [STRIFE] no such variable
 
 static boolean          headsupactive = false;
 
-// haleyjd 20130915 [STRIFE]: need nickname
-extern char *nickname;
-
 // haleyjd 20130915 [STRIFE]: true if setting nickname
 static boolean hu_setting_name = false;
 
@@ -121,7 +106,7 @@ static boolean hu_setting_name = false;
 // haleyjd 08/31/10: [STRIFE] Changed for Strife level names.
 // List of names for levels.
 
-char *mapnames[] =
+const char *mapnames[] =
 {
     // Strife map names
 
@@ -214,7 +199,7 @@ void HU_Stop(void)
 void HU_Start(void)
 {
     int         i;
-    char*       s;
+    const char *s;
 
     // haleyjd 20120211: [STRIFE] not called here.
     //if (headsupactive)
@@ -313,12 +298,12 @@ void HU_Erase(void)
 //
 //  Fastcall Registers:   edx          ebx
 //      Temp Registers:   esi          edi
-void HU_addMessage(char *prefix, char *message)
+static void HU_addMessage(const char *prefix, const char *message)
 {
     char  c;         // eax
     int   width = 0; // edx
-    char *rover1;    // ebx (in first loop)
-    char *rover2;    // ecx (in second loop)
+    const char *rover1;    // ebx (in first loop)
+    const char *rover2;    // ecx (in second loop)
     char *bufptr;    // ebx (in second loop)
     char buffer[HU_MAXLINELENGTH+2];  // esp+52h
 
@@ -546,14 +531,9 @@ boolean HU_Responder(event_t *ev)
     static boolean      altdown = false;
     unsigned char       c;
     int                 i;
-    int                 numplayers;
     
     static int          num_nobrainers = 0;
 
-    numplayers = 0;
-    for (i=0 ; i<MAXPLAYERS ; i++)
-        numplayers += playeringame[i];
-
     if (ev->data1 == KEY_RSHIFT)
     {
         return false;
