@@ -30,6 +30,7 @@
 #include "deh_main.h"
 
 #include "i_input.h"
+#include "i_joystick.h"
 #include "i_swap.h"
 #include "i_system.h"
 #include "i_timer.h"
@@ -62,13 +63,8 @@
 #include "p_dialog.h"
 
 
-extern void M_QuitStrife(int);
+void M_QuitStrife(int);
 
-extern patch_t*         hu_font[HU_FONTSIZE];
-extern boolean          message_dontfuckwithme;
-
-extern boolean          chat_on;        // in heads-up code
-extern boolean          sendsave;       // [STRIFE]
 
 //
 // defaulted values
@@ -93,7 +89,7 @@ int			quickSaveSlot;
  // 1 = message to be printed
 int			messageToPrint;
 // ...and here is the message string!
-char*			messageString;
+const char		*messageString;
 
 // message x & y
 int			messx;
@@ -131,7 +127,6 @@ boolean                 menuindialog;   // haleyjd 09/04/10: ditto
 #define CURSORXOFF		-28
 #define LINEHEIGHT		19
 
-extern boolean		sendpause;
 char			savegamestrings[10][SAVESTRINGSIZE];
 
 char	endstring[160];
@@ -146,8 +141,8 @@ short		whichCursor;		// which skull to draw
 
 // graphic name of cursors
 // haleyjd 08/27/10: [STRIFE] M_SKULL* -> M_CURS*
-char    *cursorName[8] = {"M_CURS1", "M_CURS2", "M_CURS3", "M_CURS4", 
-                          "M_CURS5", "M_CURS6", "M_CURS7", "M_CURS8" };
+const char *cursorName[8] = {"M_CURS1", "M_CURS2", "M_CURS3", "M_CURS4",
+                             "M_CURS5", "M_CURS6", "M_CURS7", "M_CURS8" };
 
 // haleyjd 20110210 [STRIFE]: skill level for menus
 int menuskill;
@@ -206,9 +201,9 @@ void M_SetupNextMenu(menu_t *menudef);
 void M_DrawThermo(int x,int y,int thermWidth,int thermDot);
 void M_DrawEmptyCell(menu_t *menu,int item);
 void M_DrawSelCell(menu_t *menu,int item);
-int  M_StringWidth(char *string);
-int  M_StringHeight(char *string);
-void M_StartMessage(char *string,void *routine,boolean input);
+int  M_StringWidth(const char *string);
+int  M_StringHeight(const char *string);
+void M_StartMessage(const char *string,void *routine,boolean input);
 void M_StopMessage(void);
 
 
@@ -547,21 +542,22 @@ void M_ReadSaveStrings(void)
 
     for(i = 0; i < load_end; i++)
     {
+        int retval;
         if(fname)
             Z_Free(fname);
         fname = M_SafeFilePath(savegamedir, M_MakeStrifeSaveDir(i, "\\name"));
 
-        handle = fopen(fname, "rb");
+        handle = M_fopen(fname, "rb");
         if(handle == NULL)
         {
-            M_StringCopy(savegamestrings[i], EMPTYSTRING,
+            M_StringCopy(savegamestrings[i], DEH_String(EMPTYSTRING),
                          sizeof(savegamestrings[i]));
             LoadMenu[i].status = 0;
             continue;
         }
-        fread(savegamestrings[i], 1, SAVESTRINGSIZE, handle);
+        retval = fread(savegamestrings[i], 1, SAVESTRINGSIZE, handle);
         fclose(handle);
-        LoadMenu[i].status = 1;
+        LoadMenu[i].status = retval == SAVESTRINGSIZE;
     }
 
     if(fname)
@@ -768,7 +764,7 @@ void M_SaveSelect(int choice)
     //saveSlot = choice;
 
     M_StringCopy(saveOldString, savegamestrings[choice], sizeof(saveOldString));
-    if (!strcmp(savegamestrings[choice],EMPTYSTRING))
+    if (!strcmp(savegamestrings[choice], DEH_String(EMPTYSTRING)))
         savegamestrings[choice][0] = 0;
     saveCharIndex = strlen(savegamestrings[choice]);
 }
@@ -1273,7 +1269,6 @@ void M_FinishReadThis(int choice)
 */
 
 #if 0
-extern void F_StartCast(void);
 
 //
 // M_CheckStartCast
@@ -1461,7 +1456,7 @@ M_DrawSelCell
 
 void
 M_StartMessage
-( char*		string,
+( const char	*string,
   void*		routine,
   boolean	input )
 {
@@ -1487,7 +1482,7 @@ void M_StopMessage(void)
 //
 // Find string width from hu_font chars
 //
-int M_StringWidth(char* string)
+int M_StringWidth(const char *string)
 {
     size_t             i;
     int             w = 0;
@@ -1510,7 +1505,7 @@ int M_StringWidth(char* string)
 //
 //      Find string height from hu_font chars
 //
-int M_StringHeight(char* string)
+int M_StringHeight(const char *string)
 {
     size_t             i;
     int             h;
@@ -1703,6 +1698,7 @@ boolean M_Responder (event_t* ev)
     static  int     lasty = 0;
     static  int     mousex = 0;
     static  int     lastx = 0;
+    int dir;
 
     // In testcontrols mode, none of the function keys should do anything
     // - the only key is escape to quit.
@@ -1746,39 +1742,70 @@ boolean M_Responder (event_t* ev)
 
     if (ev->type == ev_joystick && joywait < I_GetTime())
     {
-        if (ev->data3 < 0)
+        if (JOY_GET_DPAD(ev->data6) != JOY_DIR_NONE)
+        {
+            dir = JOY_GET_DPAD(ev->data6);
+        }
+        else if (JOY_GET_LSTICK(ev->data6) != JOY_DIR_NONE)
+        {
+            dir = JOY_GET_LSTICK(ev->data6);
+        }
+        else
+        {
+            dir = JOY_GET_RSTICK(ev->data6);
+        }
+
+        if (dir & JOY_DIR_UP)
         {
             key = key_menu_up;
             joywait = I_GetTime() + 5;
         }
-        else if (ev->data3 > 0)
+        else if (dir & JOY_DIR_DOWN)
         {
             key = key_menu_down;
             joywait = I_GetTime() + 5;
         }
-
-        if (ev->data2 < 0)
+        if (dir & JOY_DIR_LEFT)
         {
             key = key_menu_left;
-            joywait = I_GetTime() + 2;
+            joywait = I_GetTime() + 5;
         }
-        else if (ev->data2 > 0)
+        else if (dir & JOY_DIR_RIGHT)
         {
             key = key_menu_right;
-            joywait = I_GetTime() + 2;
+            joywait = I_GetTime() + 5;
         }
 
-        if (ev->data1&1)
+#define JOY_BUTTON_MAPPED(x) ((x) >= 0)
+#define JOY_BUTTON_PRESSED(x) (JOY_BUTTON_MAPPED(x) && (ev->data1 & (1 << (x))) != 0)
+
+        if (JOY_BUTTON_PRESSED(joybfire))
         {
-            key = key_menu_forward;
+            // Simulate a 'Y' keypress when Doom show a Y/N dialog with Fire button.
+            if (messageToPrint && messageNeedsInput)
+            {
+                key = key_menu_confirm;
+            }
+            else
+            {
+                key = key_menu_forward;
+            }
             joywait = I_GetTime() + 5;
         }
-        if (ev->data1&2)
+        if (JOY_BUTTON_PRESSED(joybuse))
         {
-            key = key_menu_back;
+            // Simulate a 'N' keypress when Doom show a Y/N dialog with Use button.
+            if (messageToPrint && messageNeedsInput)
+            {
+                key = key_menu_abort;
+            }
+            else
+            {
+                key = key_menu_back;
+            }
             joywait = I_GetTime() + 5;
         }
-        if (joybmenu >= 0 && (ev->data1 & (1 << joybmenu)) != 0)
+        if (JOY_BUTTON_PRESSED(joybmenu))
         {
             key = key_menu_activate;
             joywait = I_GetTime() + 5;
@@ -2291,7 +2318,7 @@ void M_Drawer (void)
     unsigned int	i;
     unsigned int	max;
     char		string[80];
-    char               *name;
+    const char          *name;
     int			start;
 
     inhelpscreens = false;
