@@ -26,6 +26,7 @@
 
 
 #include "m_bbox.h"
+#include "m_misc.h"
 
 #include "doomdef.h"
 #include "doomstat.h"
@@ -543,6 +544,7 @@ P_BlockThingsIterator
         return true;
     }
 
+    LINKED_LIST_CHECK_NO_CYCLE(mobj_t, blocklinks[y*bmapwidth+x], bnext);
 
     for (mobj = blocklinks[y*bmapwidth+x] ;
          mobj ;
@@ -772,7 +774,6 @@ P_TraverseIntercepts
     return true;            // everything was traversed
 }
 
-extern fixed_t bulletslope;
 
 #if 0
 // Intercepts Overrun emulation, from PrBoom-plus.
