@@ -26,6 +26,9 @@
 #include "p_spec.h"    // villsa [STRIFE]
 
 
+extern int numflats;
+
+
 // Retrieve column data for span blitting.
 byte*
 R_GetColumn
@@ -41,13 +44,13 @@ void R_PrecacheLevel (void);
 // Retrieval.
 // Floor/ceiling opaque texture tiles,
 // lookup by name. For animation?
-int R_FlatNumForName (char* name);
+int R_FlatNumForName(const char *name);
 
 
 // Called by P_Ticker for switches and animations,
 // returns the texture number for the texture name.
-int R_TextureNumForName (char *name);
-int R_CheckTextureNumForName (char *name);
+int R_TextureNumForName (const char *name);
+int R_CheckTextureNumForName (const char *name);
 void R_SoundNumForDoor(vldoor_t* door); // villsa [STRIFE]
 
 #endif
