@@ -148,9 +148,6 @@ static boolean          st_firsttime;
 // lump number for PLAYPAL
 static int              lu_palette;
 
-// whether in automap or first-person
-static st_stateenum_t   st_gamestate;
-
 // whether left-side main status bar is active
 static boolean          st_statusbaron;
 
@@ -201,7 +198,7 @@ static patch_t*         invsigil[5];      // sigil pieces
 static patch_t*         invarmor[2];      // armor icons
 
 // names for ammo patches
-static char *invammonames[NUMAMMO] =
+static const char *invammonames[NUMAMMO] =
 {
     "I_BLIT",
     "I_XQRL",
@@ -307,12 +304,10 @@ boolean ST_Responder(event_t* ev)
             switch(ev->data1)
             {
             case AM_MSGENTERED:
-                st_gamestate = AutomapState;
                 st_firsttime = true;
                 break;
 
             case AM_MSGEXITED:
-                st_gamestate = FirstPersonState;
                 break;
             }
 
@@ -1419,7 +1414,7 @@ boolean ST_DrawExternal(void)
     return true;
 }
 
-typedef void (*load_callback_t)(char *lumpname, patch_t **variable); 
+typedef void (*load_callback_t)(const char *lumpname, patch_t **variable);
 
 //
 // ST_loadUnloadGraphics
@@ -1483,7 +1478,7 @@ static void ST_loadUnloadGraphics(load_callback_t callback)
     callback(DEH_String("INVCURS"),  &invcursor);
 }
 
-static void ST_loadCallback(char *lumpname, patch_t **variable)
+static void ST_loadCallback(const char *lumpname, patch_t **variable)
 {
     *variable = W_CacheLumpName(lumpname, PU_STATIC);
 }
@@ -1502,7 +1497,7 @@ void ST_loadData(void)
     ST_loadGraphics();
 }
 
-static void ST_unloadCallback(char *lumpname, patch_t **variable)
+static void ST_unloadCallback(const char *lumpname, patch_t **variable)
 {
     W_ReleaseLumpName(lumpname);
     *variable = NULL;
@@ -1529,8 +1524,6 @@ void ST_initData(void)
     st_firsttime = true;
     plyr = &players[consoleplayer];
 
-    st_gamestate = FirstPersonState;
-
     st_statusbaron = true;
 
     st_palette = -1;
