@@ -68,7 +68,7 @@ static int sprite_frames_alloced;
 //
 // Returns -1 if not found
 
-static int FindInList(searchlist_t *list, char *name)
+static int FindInList(searchlist_t *list, const char *name)
 {
     int i;
 
@@ -82,8 +82,8 @@ static int FindInList(searchlist_t *list, char *name)
 }
 
 static boolean SetupList(searchlist_t *list, searchlist_t *src_list,
-                         char *startname, char *endname,
-                         char *startname2, char *endname2)
+                         const char *startname, const char *endname,
+                         const char *startname2, const char *endname2)
 {
     int startlump, endlump;
 
@@ -216,7 +216,7 @@ static sprite_frame_t *FindSpriteFrame(char *name, int frame)
     // Add to end of list
     
     result = &sprite_frames[num_sprite_frames];
-    strncpy(result->sprname, name, 4);
+    memcpy(result->sprname, name, 4);
     result->frame = frame;
 
     for (i=0; i<8; ++i)
@@ -567,7 +567,7 @@ void W_PrintDirectory(void)
 
 // Merge in a file by name
 
-void W_MergeFile(char *filename)
+void W_MergeFile(const char *filename)
 {
     int old_numlumps;
 
@@ -624,7 +624,7 @@ static void W_NWTAddLumps(searchlist_t *list)
 // Merge sprites and flats in the way NWT does with its -af and -as
 // command-line options.
 
-void W_NWTMergeFile(char *filename, int flags)
+void W_NWTMergeFile(const char *filename, int flags)
 {
     int old_numlumps;
 
@@ -670,7 +670,7 @@ void W_NWTMergeFile(char *filename, int flags)
 // a PWAD, then search the IWAD sprites, removing any sprite lumps that also
 // exist in the PWAD.
 
-void W_NWTDashMerge(char *filename)
+void W_NWTDashMerge(const char *filename)
 {
     wad_file_t *wad_file;
     int old_numlumps;
