@@ -30,8 +30,11 @@ enum
     RADIO_VALUE_MUSHROOM,
     RADIO_VALUE_SNAKE,
 };
-char *extensions[] = { "wad", "lmp", "txt", NULL };
-char *radio_values[] = { "Badger", "Mushroom", "Snake" };
+
+// also put some crazy extensions to test the escape function. a"b"c"""dd
+const char *extensions[] = { "wad", "lmp", "txt", "a\"b\"c\"\"\"dd", "",
+     "\"\"\"\"\"\"\"\"\"\"\"\"\"\"\"\"\"\"\"\"\"\"\"\"\"", NULL };
+const char *radio_values[] = { "Badger", "Mushroom", "Snake" };
 char *textbox_value = NULL;
 int numbox_value = 0;
 int radiobutton_value;
@@ -85,7 +88,7 @@ void CloseWindow(TXT_UNCAST_ARG(button), void *user_data)
 
 void UnicodeWindow(TXT_UNCAST_ARG(widget), void *user_data)
 {
-    static char *strings[] = {
+    static const char *strings[] = {
         "lunedì", "martedì", "mercoledì", "giovedì",
         "venerdì", "sabato", "domenica",
     };
