@@ -15,6 +15,7 @@
 // Routines for selecting files.
 //
 
+#include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -32,13 +33,13 @@ struct txt_fileselect_s {
     txt_widget_t widget;
     txt_inputbox_t *inputbox;
     int size;
-    char *prompt;
-    char **extensions;
+    const char *prompt;
+    const char **extensions;
 };
 
 // Dummy value to select a directory.
 
-char *TXT_DIRECTORY[] = { "__directory__", NULL };
+const char *TXT_DIRECTORY[] = { "__directory__", NULL };
 
 #ifndef _WIN32
 
@@ -155,7 +156,7 @@ int TXT_CanSelectFiles(void)
     return 0;
 }
 
-char *TXT_SelectFile(char *window_title, char **extensions)
+char *TXT_SelectFile(const char *window_title, const char **extensions)
 {
     return NULL;
 }
@@ -175,14 +176,21 @@ static BOOL (*MySHGetPathFromIDList)(LPITEMIDLIST, LPTSTR) = NULL;
 
 static int LoadDLLs(void)
 {
-    HMODULE comdlg32 = LoadLibraryW(L"comdlg32.dll");
-    HMODULE shell32 = LoadLibraryW(L"shell32.dll");
+    HMODULE comdlg32, shell32
 
-    if (comdlg32 == NULL || shell32 == NULL)
+    comdlg32 = LoadLibraryW(L"comdlg32.dll");
+    if (comdlg32 == NULL)
     {
         return 0;
     }
 
+    shell32 = LoadLibraryW(L"shell32.dll");
+    if (shell32 == NULL)
+    {
+        FreeLibrary(comdlg32);
+        return 0;
+    }
+
     MyGetOpenFileName =
         (void *) GetProcAddress(comdlg32, "GetOpenFileNameA");
     MySHBrowseForFolder =
@@ -210,7 +218,7 @@ static int InitLibraries(void)
 
 // Generate the "filter" string from the list of extensions.
 
-static char *GenerateFilterString(char **extensions)
+static char *GenerateFilterString(const char **extensions)
 {
     unsigned int result_len = 1;
     unsigned int i;
@@ -282,7 +290,7 @@ static char *SelectDirectory(char *window_title)
     return result;
 }
 
-char *TXT_SelectFile(char *window_title, char **extensions)
+char *TXT_SelectFile(const char *window_title, const char **extensions)
 {
     OPENFILENAME fm;
     char selected[MAX_PATH] = "";
@@ -331,39 +339,50 @@ char *TXT_SelectFile(char *window_title, char **extensions)
 // an Objective C dependency. This is rather silly.
 
 // Printf format string for the "wrapper" portion of the AppleScript:
+#define APPLESCRIPT_WRAPPER "copy POSIX path of (%s) to stdout"
 
-#define APPLESCRIPT_WRAPPER \
-    "tell application (path to frontmost application as text)\n" \
-    "    activate\n" \
-    "    set theFile to (%s)\n" \
-    "    copy POSIX path of theFile to stdout\n" \
-    "end tell\n"
-
-static char *EscapedString(char *s)
+static char *CreateEscapedString(const char *original)
 {
     char *result;
-    char *in, *out;
+    const char *in;
+    char *out;
 
-    result = malloc(strlen(s) + 3);
-    out = result;
-    *out++ = '\"';
-    for (in = s; *in != '\0'; ++in)
+    // We need to take care not to overflow the buffer, so count exactly.
+#define ESCAPED_CHARS "\"\\"
+    size_t count_extras = 2;    // start counting the two quotes
+    for (in = original; *in; ++in)
     {
-        if (*in == '\"' || *in == '\\')
+        if (strchr(ESCAPED_CHARS, *in))
+        {
+            ++count_extras;
+        }
+    }
+
+    result = malloc(strlen(original) + count_extras + 1);
+    if (!result)
+    {
+        return NULL;
+    }
+    out = result;
+    *out++ = '"';
+    for (in = original; *in; ++in)
+    {
+        if (strchr(ESCAPED_CHARS, *in))
         {
             *out++ = '\\';
         }
         *out++ = *in;
     }
-    *out++ = '\"';
-    *out = '\0';
+    *out++ = '"';
+    *out = 0;
 
     return result;
+#undef ESCAPED_CHARS
 }
 
 // Build list of extensions, like: {"wad","lmp","txt"}
 
-static char *ExtensionsList(char **extensions)
+static char *CreateExtensionsList(const char **extensions)
 {
     char *result, *escaped;
     unsigned int result_len;
@@ -381,16 +400,27 @@ static char *ExtensionsList(char **extensions)
     }
 
     result = malloc(result_len);
+    if (!result)
+    {
+        return NULL;
+    }
     TXT_StringCopy(result, "{", result_len);
 
     for (i = 0; extensions[i] != NULL; ++i)
     {
-        escaped = EscapedString(extensions[i]);
+        escaped = CreateEscapedString(extensions[i]);
+        if (!escaped)
+        {
+            free(result);
+            return NULL;
+        }
         TXT_StringConcat(result, escaped, result_len);
         free(escaped);
 
         if (extensions[i + 1] != NULL)
+        {
             TXT_StringConcat(result, ",", result_len);
+        }
     }
 
     TXT_StringConcat(result, "}", result_len);
@@ -398,30 +428,39 @@ static char *ExtensionsList(char **extensions)
     return result;
 }
 
-static char *GenerateSelector(char *window_title, char **extensions)
+static char *GenerateSelector(const char *const window_title, const char **extensions)
 {
-    char *chooser, *ext_list, *result;
-    unsigned int result_len;
-
-    result_len = 64;
+    const char *chooser;
+    char *ext_list = NULL;
+    char *window_title_escaped = NULL;
+    char *result = NULL;
+    unsigned int result_len = 64;
 
     if (extensions == TXT_DIRECTORY)
     {
         chooser = "choose folder";
-        ext_list = NULL;
     }
     else
     {
         chooser = "choose file";
-        ext_list = ExtensionsList(extensions);
+        ext_list = CreateExtensionsList(extensions);
+        if (!ext_list)
+        {
+            return NULL;
+        }
     }
 
     // Calculate size.
 
     if (window_title != NULL)
     {
-        window_title = EscapedString(window_title);
-        result_len += strlen(window_title);
+        window_title_escaped = CreateEscapedString(window_title);
+        if (!window_title_escaped)
+        {
+            free(ext_list);
+            return NULL;
+        }
+        result_len += strlen(window_title_escaped);
     }
     if (ext_list != NULL)
     {
@@ -429,35 +468,51 @@ static char *GenerateSelector(char *window_title, char **extensions)
     }
 
     result = malloc(result_len);
+    if (!result)
+    {
+        free(window_title_escaped);
+        free(ext_list);
+        return NULL;
+    }
 
     TXT_StringCopy(result, chooser, result_len);
 
-    if (window_title != NULL)
+    if (window_title_escaped != NULL)
     {
         TXT_StringConcat(result, " with prompt ", result_len);
-        TXT_StringConcat(result, window_title, result_len);
-        free(window_title);
+        TXT_StringConcat(result, window_title_escaped, result_len);
     }
 
     if (ext_list != NULL)
     {
-        TXT_StringConcat(result, "of type ", result_len);
+        TXT_StringConcat(result, " of type ", result_len);
         TXT_StringConcat(result, ext_list, result_len);
-        free(ext_list);
     }
 
+    free(window_title_escaped);
+    free(ext_list);
     return result;
 }
 
-static char *GenerateAppleScript(char *window_title, char **extensions)
+static char *GenerateAppleScript(const char *window_title, const char **extensions)
 {
     char *selector, *result;
     size_t result_len;
 
     selector = GenerateSelector(window_title, extensions);
+    if (!selector)
+    {
+        return NULL;
+    }
 
     result_len = strlen(APPLESCRIPT_WRAPPER) + strlen(selector);
     result = malloc(result_len);
+    if (!result)
+    {
+        free(selector);
+        return NULL;
+    }
+
     TXT_snprintf(result, result_len, APPLESCRIPT_WRAPPER, selector);
     free(selector);
 
@@ -469,12 +524,16 @@ int TXT_CanSelectFiles(void)
     return 1;
 }
 
-char *TXT_SelectFile(char *window_title, char **extensions)
+char *TXT_SelectFile(const char *window_title, const char **extensions)
 {
     char *argv[4];
     char *result, *applescript;
 
     applescript = GenerateAppleScript(window_title, extensions);
+    if (!applescript)
+    {
+        return NULL;
+    }
 
     argv[0] = "/usr/bin/osascript";
     argv[1] = "-e";
@@ -495,7 +554,7 @@ char *TXT_SelectFile(char *window_title, char **extensions)
 
 #define ZENITY_BINARY "/usr/bin/zenity"
 
-static unsigned int NumExtensions(char **extensions)
+static unsigned int NumExtensions(const char **extensions)
 {
     unsigned int result = 0;
 
@@ -523,7 +582,7 @@ int TXT_CanSelectFiles(void)
 // return a pointer to a string that is a case-insensitive
 // pattern representation (like [Ww][Aa][Dd])
 //
-static char *ExpandExtension(char *orig)
+static char *ExpandExtension(const char *orig)
 {
     int oldlen, newlen, i;
     char *c, *newext = NULL;
@@ -556,7 +615,7 @@ static char *ExpandExtension(char *orig)
     return newext;
 }
 
-char *TXT_SelectFile(char *window_title, char **extensions)
+char *TXT_SelectFile(const char *window_title, const char **extensions)
 {
     unsigned int i;
     size_t len;
@@ -570,8 +629,8 @@ char *TXT_SelectFile(char *window_title, char **extensions)
     }
 
     argv = calloc(5 + NumExtensions(extensions), sizeof(char *));
-    argv[0] = ZENITY_BINARY;
-    argv[1] = "--file-selection";
+    argv[0] = strdup(ZENITY_BINARY);
+    argv[1] = strdup("--file-selection");
     argc = 2;
 
     if (window_title != NULL)
@@ -611,7 +670,7 @@ char *TXT_SelectFile(char *window_title, char **extensions)
 
     result = ExecReadOutput(argv);
 
-    for (i = 2; i < argc; ++i)
+    for (i = 0; i < argc; ++i)
     {
         free(argv[i]);
     }
@@ -757,7 +816,7 @@ static void InputBoxChanged(TXT_UNCAST_ARG(widget), TXT_UNCAST_ARG(fileselect))
 }
 
 txt_fileselect_t *TXT_NewFileSelector(char **variable, int size,
-                                      char *prompt, char **extensions)
+                                      const char *prompt, const char **extensions)
 {
     txt_fileselect_t *fileselect;
 
